#ifndef CHROMEOS_PLATFORMS_CAMERA_LIBHDR_GCAM_AE_H_
#define CHROMEOS_PLATFORMS_CAMERA_LIBHDR_GCAM_AE_H_

#include <cstdint>
#include <memory>
#include <vector>

namespace cros {

// A normalized rectangle for a image with top-left (x0, y0) and bottom-right
// (x1, y1) coordinates in [0, 1].
struct NormalizedRect {
  float x0 = 0.0f;
  float x1 = 1.0f;
  float y0 = 0.0f;
  float y1 = 1.0f;
};

// AeResultCrOS carries the total exposure times (TETs) produced by the Gcam
// auto-exposure algorithm.
struct AeResultCrOS {
  // `short_tet` is the actual TET that should be applied to the sensor. Usually
  // we shorten the exposure time to avoid clipping the highlight areas.
  float short_tet = 0.0f;
  // `long_tet` is the desired exposure time for the shadow areas on the image,
  // and is used to produce the HDR ratio for the HDRnet pipeline.
  float long_tet = 0.0f;

  bool IsValid() const { return short_tet > 0.0f && long_tet > 0.0f; }
};

// AeFrameMetadataCrOS is a light-weight replica of
// gcam::cameralite::AeFrameMetadata and carries the input parameters to the
// Gcam AE algorithm.
struct AeFrameMetadataCrOS {
  float actual_analog_gain = 0.0f;
  float applied_digital_gain = 0.0f;
  float actual_exposure_time_ms = 0.0f;
  float sensor_sensitivity = 0.0f;
  std::vector<NormalizedRect> faces;
};

// YuvFormat defines the YUV formats that can be accepted by the Gcam AE
// algorithm. Currently NV12 is the only supported format.
enum class YuvFormat {
  kNv12,
};

// YUV buffer metadata of a YUV buffer. Currently the only supported YUV format
// is NV12 hence `planes` has only size 2.
struct YuvBuffer {
  YuvFormat format;
  int width;
  int height;
  struct {
    int width;
    int height;
    int stride;
    const uint8_t* data;
  } planes[2];
};

class GcamAe {
 public:
  // Factory method to create a GcamAe instance.
  static std::unique_ptr<GcamAe> CreateInstance();

  // Computes a set of exposure parameters from the input `metadata`,
  // `yuv_buffer`, and `hdr_ratio`.
  virtual AeResultCrOS ComputeLinearizedGcamAe(
      const AeFrameMetadataCrOS& metadata, const YuvBuffer& yuv_buffer,
      float max_hdr_ratio) = 0;

  virtual ~GcamAe() = default;
};

}  // namespace cros

#endif  // CHROMEOS_PLATFORMS_CAMERA_LIBHDR_GCAM_AE_H_
